/* global __dirname */
const CleanWebpackPlugin = require('clean-webpack-plugin');
const CopyWebpackPlugin = require('copy-webpack-plugin');
const ExtractCssChunks = require('extract-css-chunks-webpack-plugin');
const HtmlWebpackPlugin = require('html-webpack-plugin');
const OptimizeCSSAssetsPlugin = require('optimize-css-assets-webpack-plugin');
const ScriptExtHtmlWebpackPlugin = require('script-ext-html-webpack-plugin');
const SriPlugin = require('webpack-subresource-integrity');
const StyleLintPlugin = require('stylelint-webpack-plugin');
const TerserPlugin = require('terser-webpack-plugin');
const WebappWebpackPlugin = require('webapp-webpack-plugin');
const merge = require('webpack-merge');
const path = require('path');
const webpack = require('webpack');
const walkSync = require('./util/walkSync.js');

let pages = [];
try {
  walkSync('build/pages/').forEach(file => {
    if (file.endsWith('.html')) {
      let page = new HtmlWebpackPlugin({
        // Also generate a test.html
        filename: `${file.replace('build/pages/', '')}`,
        template: `build/pages/${file.replace('build/pages/', '')}`,
        inject  : 'head',
        xhtml   : true,
      });
      pages.push(page);
    }
  });
} catch (error) {
  throw error;
}

const commonConfig = merge([
  {
    mode   : 'production',
    devtool: 'cheap-source-map',
    stats  : {
      'all'         : false,
      'assets'      : false,
      'colors'      : true,
      'errors'      : true,
      'warnings'    : true,
      'errorDetails': true,
    },
    entry    : {'uark-main': path.resolve(__dirname, 'src/js/index.js')},
    externals: {
      bootstrap: 'jQuery',
      jquery   : 'jQuery',
      popper   : 'Popper',
    },
    output: {
      filename          : 'js/[name]-[contenthash].pack.js',
      chunkFilename     : 'js/[name]-[id][contenthash].pack.js',
      path              : path.resolve(__dirname, 'public'),
      publicPath        : '',
      crossOriginLoading: 'anonymous',
      hashDigestLength  : 12,
    },
    plugins: [
      new CleanWebpackPlugin([ 'public' ]),
      new HtmlWebpackPlugin({
        filename: 'statics.html',
        template: 'src/xsl/statics.html',
        inject  : 'head',
        xhtml   : true,
      }),
      ...pages,
      new ScriptExtHtmlWebpackPlugin({defaultAttribute: 'defer'}),
      new ExtractCssChunks({
        // Options similar to the same options in webpackOptions.output
        // both options are optional
        filename     : 'css/[name]-[contenthash].pack.css',
        chunkFilename: 'css/[name]-[id][contenthash].pack.css',

        hot         : false, // if you want HMR - we try to automatically inject hot reloading but if it's not working, add it to the config
        orderWarning: true, // Disable to remove warnings about conflicting order between imports
        reloadAll   : false, // when desperation kicks in - this is a brute force HMR flag
        cssModules  : true, // if you use cssModules, this can help.
      }),
      new CopyWebpackPlugin([
        {
          from  : path.resolve(__dirname, 'src/assets'),
          to    : 'assets/[name].[hash].[ext]',
          toType: 'template',
        },
      ]),
      new CopyWebpackPlugin([
        {
          from   : '**/*.+(jpg|png|svg)',
          to     : '',
          context: 'src/content',
        },
      ]),
      new WebappWebpackPlugin({
        logo    : path.resolve(__dirname, 'src/assets/icons/shield.png'),
        prefix  : 'assets/icons/',
        favicons: {
          title              : null,
          appName            : null,
          description        : null,
          appleStatusBarStyle: 'black-translucent', // Style for Apple status bar: "black-translucent", "default", "black". `string`
          display            : 'browser', // Preferred display mode: "fullscreen", "standalone", "minimal-ui" or "browser". `string`
          background         : '#9d2235',
          icons              : {
            android     : true,
            appleIcon   : true,
            appleStartup: false,
            coast       : false,
            favicons    : true,
            firefox     : false,
            opengraph   : true,
            twitter     : false,
            yandex      : false,
            windows     : false,
          },
        },
      }),
    ],
    module: {rules: [
      {
        test   : /\.jsx?$/,
        include: path.resolve(__dirname, 'src/js'),
        use    : {
          loader : 'babel-loader',
          options: {cacheDirectory: '.babelcache'},
        },
      },
      {
        test   : /\.(svg|png|jpg)$/,
        loader : 'file-loader?limit=8192',
        include: path.resolve(__dirname, 'src/assets'),
        options: {
          // emitFile: true,
          name      : '[name].[ext]',
          outputPath: 'assets',
        },
      },
      {
        test   : /\.sass$|\.scss$|\.css$/,
        include: path.resolve(__dirname, 'src/sass'),
        use    : [
          {
            loader : ExtractCssChunks.loader,
            options: {sourceMap: true},
          },
          {
            loader : 'css-loader',
            options: {
              importLoaders: 1,
              sourceMap    : true,
            },
          },
          {
            loader : 'postcss-loader',
            options: {sourceMap: true},
          },
          {
            loader : 'sass-loader',
            options: {sourceMap: true},
          },
        ],
      },
      {
        test   : /\.css$/,
        include: path.resolve(__dirname, 'node_modules'),
        use    : [
          {
            loader : ExtractCssChunks.loader,
            options: {sourceMap: true},
          },
          {
            loader : 'css-loader',
            options: {
              importLoaders: 1,
              sourceMap    : true,
            },
          },
        ],
      },

    ]},
    optimization: {
      minimizer: [
        new TerserPlugin({
          cache    : '.tersercache',
          parallel : true,
          sourceMap: true,
        }),
        new OptimizeCSSAssetsPlugin({
          cssProcessorOptions: {
            map: {
              inline    : false,
              annotation: true,
            },
            sourceMap: true,
            level    : {1: {
              all          : true,
              normalizeUrls: false,
            }},

          },
          canPrint: true,
        }),
      ],
      runtimeChunk: 'single',
      splitChunks : {cacheGroups: {
        cicd: {
          test    : /[\\/]node_modules[\\/](prismjs|clipboard)[\\/]/,
          name    : 'cicd',
          chunks  : 'all',
          enforce : true,
          priority: 10,
        },
        cicdcss: {
          test    : /[\\/]src[\\/]sass[\\/](uark-examples)\.scss/,
          name    : 'cicd',
          chunks  : 'all',
          enforce : true,
          priority: 10,
        },
        extra: {
          test    : /[\\/]node_modules[\\/](hideseek)[\\/]/,
          name    : 'extra',
          chunks  : 'all',
          enforce : true,
          priority: 10,
        },
        vendor: {
          test  : /[\\/]node_modules[\\/]/,
          name  : 'vendors',
          chunks: 'all',
        },
      }},
    },
  },
]);

const productionConfig = merge([
  {plugins: [
    new SriPlugin({
      hashFuncNames: [ 'sha256', 'sha384' ],
      enabled      : true,
    }),
    new webpack.HashedModuleIdsPlugin(),
  ]},
]);

const developmentConfig = merge([
  {
    output      : {publicPath: '/'},
    mode        : 'development',
    devtool     : 'source-map',
    watch       : false,
    // webpack-dev-server turns 'watch' on. Setting false here lets us do a manual development mode pack
    watchOptions: {
      ignored: [
        /node_modules/,
        /public/,
        /src[\\/](content|templates|xsl)/,
      ],
      aggregateTimeout: 1500,
    },
    devServer: {
      contentBase: path.join(__dirname, 'public'),
      stats      : {
        'all'         : false,
        'colors'      : true,
        'errors'      : true,
        'warnings'    : true,
        'errorDetails': true,
      },
      host    : 'localhost',
      port    : 8080,
      open    : true,
      openPage: 'index.html',
      overlay : {
        warnings: true, errors  : true,
      },
    },
    plugins: [
      new StyleLintPlugin({
        configFile : '.stylelintrc.json',
        emitErrors : false,
        failOnError: false,
        files      : 'src/sass/*.scss',
        fix        : true,
        syntax     : 'scss',

      }),
    ],
    module: {rules: [
      {
        test   : /\.jsx?$/,
        enforce: 'pre',
        loader : 'eslint-loader',
        exclude: /node_modules/,
        options: {
          emitWarning: true,
          emitError  : true,
          configFile : './.eslintrc.json',
          cache      : true,
          fix        : true,
        },
      },
    ]},
  },
]);

module.exports = (env) => {
  if (env.production) {
    return merge(commonConfig, productionConfig, {
      mode  : 'production',
      output: {publicPath: env.publicPath || '/'},
    });
  }

  return merge(commonConfig, developmentConfig, {mode: 'development'});
};
